//// textures
texture mytex;
sampler2D mySamp = sampler_state // This sampler struct is used to specify texture options.
{
       Texture = (mytex);
       MinFilter = NONE;
       MagFilter = NONE;
       MipFilter = NONE;

       AddressU = Clamp;
       AddressV = Clamp;
};

texture mytexTemp;
sampler1D mySampTemp = sampler_state // This sampler struct is used to specify texture options.
{
       Texture = (mytexTemp);
       MinFilter = NONE;
       MagFilter = NONE;
       MipFilter = NONE;

       AddressU = Clamp;
       AddressV = Clamp;
};

//// inputs, outputs
uniform float4 pivotRaw : COLOR1;  // Pivot passed from DX
// uniform float4 pivotPos : COLOR2;  // Pivot passed from DX precomputed

struct VS_OUTPUT // This structure will define how we are sending the data to the pixel shader.
{ 
    float4 Pos: POSITION;
	float2 Tex0: TEXCOORD0;
};

struct PS_OUTPUT // This final struct will define what output the pixel shader will produce.
{
	float4 Color: COLOR;
};

PS_OUTPUT psSubMat (VS_OUTPUT In) : COLOR // inMatrix[j+i*width] = inMatrix[j+i*width] - tmpMatrix[t+i*width] * inMatrix[j+t*width];
{
	PS_OUTPUT Out;
	float inMatA,tmpMat,inMatB;
	float pivotY,pivotYplus,pivotXminus;
	float tmp;

	pivotY=(pivotRaw.a+0.5)/pivotRaw.g; // pivot/height
	pivotXminus=(pivotRaw.a)/pivotRaw.r; // pivot/width
	pivotYplus=(pivotRaw.a+1)/pivotRaw.g; // pivot/height

	inMatA = tex2D(mySamp, In.Tex0).r; // inMatrix[j+i*width]
	inMatB = tex2D(mySamp, float2(In.Tex0.x, pivotY)).r; // inMatrix[j+t*width]
	tmpMat = tex1D(mySampTemp, In.Tex0.y).r; // tmpMatrix[t+i*width]
  
    // for (int i=(t+1); i<height; i++) for (int j=t; j<width; j++)
	if (In.Tex0.y>pivotYplus && In.Tex0.x>pivotXminus) // submat
	{
//		if (pivotPosRaw.a == 2)
//			inMatA=inMatB;
//		else	
			tmp = tmpMat*inMatB;
			inMatA = inMatA-tmp;
//			inMatA = tmpMat*inMatB; // DEBUG
	} else // original
		inMatA = tex2D(mySamp, In.Tex0).r;
		
//	inMatA = tex2D(mySamp, In.Tex0).r; // DEBUG
//	inMatA = 1; // DEBUG

	Out.Color = ( 0, 0, 0, 0 );
	Out.Color.r = inMatA; // inMatrix[j+i*width]

	return Out;
}

PS_OUTPUT psRow (VS_OUTPUT In) : COLOR // tmpMatrix[t+i*width] = inMatrix[t+i*width] / pivot;
{
	PS_OUTPUT Out;
	float pivot,inMat;
	float pivotX,pivotY;
	float tmp, tmp2;
	
	pivotX = (pivotRaw.a+0.5)/pivotRaw.r; // pivot/width
	pivotY = (pivotRaw.a+0.5)/pivotRaw.g; // pivot/height
	
	inMat = tex2D(mySamp, float2(pivotX,In.Tex0.y)).r; // inMatrix[t+i*width]
	pivot = tex2D(mySamp, float2(pivotX,pivotY)).r; // inMatrix[t+t*width]

	Out.Color = ( 0, 0, 0, 0 );
	// for (int i=(t+1); i<height; i++)
//	if (In.Tex0.y>pivotPos.a) // row
		Out.Color.r = inMat / pivot; // tmpMatrix[t+i*width]
//	else // otherwise doesn't matter
//		Out.Color.r = tex2D(mySamp, In.Tex0); // inMatrix[j+i*width]

//	Out.Color.r = tex2D(mySamp, In.Tex0); // DEBUG

	return Out;
}

PS_OUTPUT psMulMat (VS_OUTPUT In) : COLOR // Testing purposes (only copy value)
{
	PS_OUTPUT Out;
	Out.Color = ( 0, 0, 0, 0 );
	Out.Color.r = tex2D(mySamp, In.Tex0.xy).r * pivotRaw.a;
	
	return Out;
}

PS_OUTPUT psTest (VS_OUTPUT In) : COLOR // Testing purposes (only copy value)
{
	PS_OUTPUT Out;
	float a, b;
	float resXA, resXB, resXminus;

	Out.Color = ( 0, 0, 0, 0 );	

	resXA=(pivotRaw.r-3.5)/pivotRaw.r;
	resXB=(pivotRaw.r-2.5)/pivotRaw.r;
	resXminus=(pivotRaw.r-2)/pivotRaw.r; // pivot/width
	
	a=tex2D(mySamp, float2(resXA, In.Tex0.y)); // width-4
	b=tex2D(mySamp, float2(resXB, In.Tex0.y)); // width-3

	if (In.Tex0.x>resXminus)
		Out.Color.r = ( a / b );
	else
		Out.Color.r = tex2D(mySamp, In.Tex0.xy);

	return Out;
}

//// Technique
technique poctySubMat
{
    pass P0
    {
		// Here we will compile and set the vertex and pixel shader.
		VertexShader = NULL; // Use fixed-function vertex pipeline
        PixelShader = compile ps_2_0 psSubMat ();
    }
}

technique poctyRow
{
    pass P0
    {
		// Here we will compile and set the vertex and pixel shader.
		VertexShader = NULL; // Use fixed-function vertex pipeline
        PixelShader = compile ps_2_0 psRow ();
    }
}

technique poctyTest
{
	pass P0
	{
		VertexShader = NULL;
		PixelShader = compile ps_2_0 psTest();
	}
}

technique poctyMulMat
{
	pass P0
	{
		VertexShader = NULL;
		PixelShader = compile ps_2_0 psMulMat();
	}
}